import { Result } from "jalhyd";

import { ServiceFactory } from "../services/service-factory";
import { MultiDimensionResults } from "./multidimension-results";
import { PbCloisonResults } from "./pb-cloison-results";

export class PrebarrageResults extends MultiDimensionResults {

    /** résultats des bassins, dans l'ordre */
    public bassinsResults: Result[];

    /** résultat de la cloison actuellement sélectionnée (nourrit le FixedResultsComponent) */
    public cloisonResults: PbCloisonResults;

    /** symboles des colonnes de résultat */
    protected _columns: string[];

    /** size of the longest variating parameter */
    public size: number;

    /** Stores appropriate number suffix for a given wall uid (copied from PbSchema) */
    public wallsSuffixes: { [key: string]: number } = {};

    public constructor() {
        super();
        this.reset();
        this.size = 1; // boulette-proufe
        // standard headers
        this._columns = [
            "BASSIN",
            "S",
            "ZF",
            "Z",
            "PV",
            "YMOY",
            "Q"
        ];
    }

    /** headers symbols */
    public get columns() {
        return this._columns;
    }

    /** translated headers texts */
    public get headers() {
        return this._columns.map((h) => {
            // calculator type for translation
            const sn = this.result.sourceNub;
            let ct = sn.calcType;
            if (sn.parent) {
                ct = sn.parent.calcType;
            }
            let label = ServiceFactory.formulaireService.expandVariableNameAndUnit(ct , h);
            label += this.getHelpLink(h);
            return label;
        });
    }

    // redéfinir le set() implique de redéfinir le get(), sinon /i
    public get variableIndex(): number {
        return this._variableIndex;
    }

    public set variableIndex(v: number) {
        this._variableIndex = v;
        // set index in pseudo-fixed Cloison results too
        this.cloisonResults.variableIndex = v;
    }

    public reset() {
        super.reset();
        this.bassinsResults = [];
        this.cloisonResults = new PbCloisonResults();
        this.cloisonResults.variableIndex = this._variableIndex;
        this.cloisonResults.size = this.size;
        this.result = undefined;
    }

    /**
     * Returns true if at least one log message is present in the PAB result or any
     * of the children results
     */
    public get hasLog(): boolean {
        if (this.bassinsResults) {
            for (const cr of this.bassinsResults) {
                if (cr && cr.hasLog()) {
                    return true;
                }
            }
        }
        return (this.cloisonResults && this.cloisonResults.result && this.cloisonResults.result.hasLog());
    }

    public get hasResults(): boolean {
        return this.hasBasinResults || this.hasWallResults;
    }

    public get hasBasinResults(): boolean {
        return this.bassinsResults.length > 0 && this.bassinsResults[0] && ! this.bassinsResults[0].hasOnlyErrors;
    }

    public get hasWallResults(): boolean {
        return this.cloisonResults  && this.cloisonResults.result && ! this.cloisonResults.result.hasOnlyErrors;
    }

    /** retourne true si au moins un calcul a échoué (le log a un code négatif) */
    public hasError(): boolean {
        let err = false;
        // log principal
        err = (err || (this.cloisonResults && this.cloisonResults && this.cloisonResults.result.hasErrorMessages()));
        // logs des bassins
        for (const c of this.bassinsResults) {
            err = (err || c.hasErrorMessages());
        }

        return err;
    }

    /** retourne true si le calcul à l'itération i a échoué */
    public iterationHasError(i: number): boolean {
        let err = this.cloisonResults && this.cloisonResults.result && this.cloisonResults.result.resultElements[i].hasErrorMessages();
        // logs des bassins
        for (const c of this.bassinsResults) {
            err = (err || c.resultElements[i].hasErrorMessages());
        }

        return err;
    }

    /** retourne true si tous les calculs ont échoué */
    public hasOnlyErrors(): boolean {
        let err = true;
        // log principal
        err = (err && this.cloisonResults && this.cloisonResults.result && this.cloisonResults.result.hasOnlyErrors);
        // logs des bassins
        for (const c of this.bassinsResults) {
            err = (err && c.hasOnlyErrors);
        }

        return err;
    }
}
